COMPAIR_MODIFIED_ATTRIBUTES equ 0x1
COMPAIR_MODIFIED_DATE equ 0x2
COMPAIR_MODIFIED_SIZE equ 0x4
COMPAIR_MODIFIED_BITWISE equ 0x8
;COMPAIR_MODIFIED_HASH equ 0x10

COMPAIR_REGISTER_ADDED equ 0x0
COMPAIR_REGISTER_DELETED equ 0x20 ;switches new and old when calling registerFile
COMPAIR_REGISTER_MODIFIED equ 0x40

COMPAIR_DIR1 equ COMPAIR_MODIFIED_ATTRIBUTES or \
COMPAIR_MODIFIED_DATE or COMPAIR_MODIFIED_SIZE or \
COMPAIR_REGISTER_ADDED or COMPAIR_REGISTER_MODIFIED


COMPAIR_DIR2 equ COMPAIR_REGISTER_DELETED

align 0x10
Done db 0
Working db 0
StopDirLoop db 0

align 0x10
Path pointer 0
Path2 pointer 0



CurrentFileMemLock pointer 0
CurrentFileMem pointer 0
CurrentFolderMemLock pointer 0
CurrentFolderMem pointer 0

compareQwords:
  ;counters for compair dir1 with dir2
  CountedDirs dq 0
  CountedFiles dq 0
  SizeCount dq 0
  TotalErrPath dq 0

  ;counters for compair dir2 with dir1
  CountedDirs2 dq 0
  CountedFiles2 dq 0
  SizeCount2 dq 0
  TotalErrPath2 dq 0

  SizeToBackup dq 0            ;All the size to copy,
  FilesToRemove dq 0           ;The amount of files that need to be removed
  FilesToChangeAttributes dq 0 ;The amount of files that the attributes need to be removed

  backupedSize dq 0
  removedFiles dq 0
  changedAttributesFiles dq 0

  changedResults dq 0
endCompareQwords:

cStarted dq 0   ;amount of times 'Start' is called

;startCollections:
;pointer 0
;endCollections:

startDwordCollections:
  ;collections for compair dir1 with dir2
  Modified_Attributes_files pointer 0     ;path and filesnames of files with changed attributes
  Modified_TimeDate_files pointer 0       ;path and filesnames of files with changed datetime stamp
  Modified_Size_files pointer 0           ;path and filesnames of files with changed size
  no_access_files pointer 0
  Added_files pointer 0                   ;path and filesnames of files that are new

  ;collections for compair dir2 with dir1
  no_access_files2 pointer 0
  Deleted_files pointer 0                 ;path and filesnames of files that where deleted in the original path

  ;collections for compair dir1 with dir2
  dModified_Attributes_files pointer 0         ;old and new attributes
  dModified_TimeDate_files pointer 0           ;old and new Modified timedate
  dModified_TimeDate_files_size pointer 0      ;size of the Modified timedate files
  dModified_Size_files pointer 0               ;old and new size
  dno_access_files pointer 0                   ;old size
  dAdded_files pointer 0                       ;old and new size

  ;collections for compair dir2 with dir1
  dno_access_files2 pointer 0                  ;new size
  dDeleted_files pointer 0                     ;old size

  Modified_Attributes_files_selected pointer 0 ;Selections which files to change attributes from
  Modified_TimeDate_files_selected pointer 0   ;Selections which files to copy
  Modified_Size_files_selected pointer 0       ;Selections which files to copy
  Added_files_selected pointer 0               ;Selections which files to copy
  Deleted_files_selected pointer 0             ;Selections which files to remove

  Modified_Attributes_files_result pointer 0 ;results of changed attributes
  Modified_TimeDate_files_result pointer 0   ;results of changed modified timeDate
  Modified_Size_files_result pointer 0       ;results of changed modified size
  Added_files_result pointer 0               ;results of added files
  Deleted_files_result pointer 0             ;results of deleted_files
endDwordCollections:

macro tryLockDontWait pointer, jumpFail
{ common
  local ..lockIsSet

  push pecx
  push peax

  if pointer eq peax | pointer eq pecx
    push pedx
    mov pedx,pointer
  end if

    ;for optimalisation, try cmp first, without lock
    xor al,al
    if pointer eq peax | pointer eq pecx
      cmp byte [pedx],al
    else
      cmp byte [pointer],al
    end if
    jne ..lockIsSet

    mov cl,1
    if pointer eq peax | pointer eq pecx
      lock cmpxchg byte [pedx],cl
    else
      lock cmpxchg byte [pointer],cl
    end if

  ..lockIsSet:
  if pointer eq peax | pointer eq pecx
    pop pedx
  end if

  pop peax
  pop pecx

  jnz jumpFail
}



;------------------------------------------------------------------------------
;This function will start comparing two directories
;input:
;    1:reserved

;output:
;       None
;------------------------------------------------------------------------------
StarCompare:
;Create param

;lock the AntiWriteLockCollection
push FULL_LOCK
push [dModified_TimeDate_files_size] ;modified timedate size of file dword collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [dno_access_files]           ;No access dirs collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [no_access_files]            ;No access dirs collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [dModified_Attributes_files] ;modified new-old attributes dword collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [dModified_TimeDate_files]   ;modified new-old timedate dword collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [dModified_Size_files]       ;modified  new-old size dword collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [dno_access_files2]          ;No access file  new-old size dword collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [dAdded_files]               ;Added files new-old size collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [Modified_Attributes_files] ;modified attributes files collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [Modified_TimeDate_files]   ;modified timedate files collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [Modified_Size_files]       ;modified size files collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [no_access_files2]           ;No access files collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [Added_files]               ;Added files collection
call LockAntiWriteLockCollection

push [dModified_TimeDate_files_size] ;modified timedate size of file dword collection
push [dno_access_files]           ;No access dirs collection
push [no_access_files]            ;No access dirs collection
push [dModified_Attributes_files] ;modified new-old attributes dword collection
push [dModified_TimeDate_files]   ;modified new-old timedate dword collection
push [dModified_Size_files]       ;modified  new-old size dword collection
push [dno_access_files2]          ;No access file  new-old size dword collection
push [dAdded_files]               ;Added files new-old size collection

push [Modified_Attributes_files] ;modified attributes files collection
push [Modified_TimeDate_files]   ;modified timedate files collection
push [Modified_Size_files]       ;modified size files collection
push [no_access_files2]           ;No access files collection
push [Added_files]               ;Added files collection
push COMPAIR_DIR1                ;flags
push [Path2]                     ;compair path to
push [Path]                      ;Base path
push TotalErrPath                ;Total unable to open directories
push [CurrentFileMem]            ;Current working file
push [CurrentFolderMem]          ;Current working dir
push CountedFiles                ;Total counted files
push CountedDirs                 ;Total counted dirs
push SizeCount                   ;Total size
mov pedx,pesp

push StopDirLoop   ;stop pointer
push 2             ;first dirs, then files
push pedx          ;param
push errDir        ;callback when error reading directory
push newSubdir     ;callback when a new directory is being entered
push countFiles    ;callback function
push [Path]        ;Path
call StartDirectoryLoop  ;startComparing


add pesp,psize*22

push FULL_LOCK
push [dModified_TimeDate_files_size] ;modified timedate size of file dword collection
call ExternalUnlockAntiWriteLockCollection

push [dModified_TimeDate_files_size] ;modified timedate size of file dword collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [dModified_Attributes_files] ;modified new-old attributes dword collection
call ExternalUnlockAntiWriteLockCollection

push [dModified_Attributes_files] ;modified new-old attributes dword collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [dModified_TimeDate_files]   ;modified new-old timedate dword collection
call ExternalUnlockAntiWriteLockCollection

push [dModified_TimeDate_files]   ;modified new-old timedate dword collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [dModified_Size_files]       ;modified  new-old size dword collection
call ExternalUnlockAntiWriteLockCollection

push [dModified_Size_files]       ;modified  new-old size dword collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [dAdded_files]               ;Added files new-old size collection
call ExternalUnlockAntiWriteLockCollection

push [dAdded_files]               ;Added files new-old size collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [Modified_Attributes_files] ;modified attributes files collection
call ExternalUnlockAntiWriteLockCollection

push [Modified_Attributes_files] ;modified attributes files collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [Modified_TimeDate_files]   ;modified timedate files collection
call ExternalUnlockAntiWriteLockCollection

push [Modified_TimeDate_files]   ;modified timedate files collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [Modified_Size_files]       ;modified size files collection
call ExternalUnlockAntiWriteLockCollection

push [Modified_Size_files]       ;modified size files collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [Added_files]               ;Added files collection
call ExternalUnlockAntiWriteLockCollection

push [Added_files]               ;Added files collection
call MergeAntiWriteLockCollectionWithCollection

push FULL_LOCK
push [Deleted_files]           ;No access files collection
call LockAntiWriteLockCollection

push FULL_LOCK
push [dDeleted_files]               ;Added files collection
call LockAntiWriteLockCollection


cmp [StopDirLoop],1
je stopStarCompare

;----------------------

push 0                            ;modified timedate size of file dword collection
push [dno_access_files2]          ;No access dirs collection
push [no_access_files2]           ;No access dirs collection
push 0                            ;modified attributes files collection
push 0                            ;modified timedate files collection
push 0                            ;modified size files collection
push [dno_access_files]           ;No access files collection
push [dDeleted_files]              ;Added files collection

push 0                            ;modified attributes files collection
push 0                            ;modified timedate files collection
push 0                            ;modified size files collection
push [no_access_files]           ;No access files collection
push [Deleted_files]              ;Added files collection
push COMPAIR_DIR2  ;flags
push [Path]       ;compair path to
push [Path2]        ;Base path
push TotalErrPath2  ;Total unable to open directories
push [CurrentFileMem]            ;Current working file
push [CurrentFolderMem]          ;Current working dir
push CountedFiles2  ;Total counted files
push CountedDirs2   ;Total counted dirs
push SizeCount2     ;Total size
mov pedx,pesp

push StopDirLoop   ;stop pointer
push 2             ;first dirs, then files
push pedx          ;param
push errDir        ;callback when error reading directory
push newSubdir     ;callback when a new directory is being entered
push countFiles    ;callback function
push [Path2]       ;Path
call StartDirectoryLoop

add pesp,psize*22

stopStarCompare:
;unlock the AntiWriteLockCollection


push FULL_LOCK
push [dDeleted_files]
call ExternalUnlockAntiWriteLockCollection

push FULL_LOCK
push [Deleted_files]
call ExternalUnlockAntiWriteLockCollection

push FULL_LOCK
push [dno_access_files]           ;No access dirs collection
call ExternalUnlockAntiWriteLockCollection

push FULL_LOCK
push [no_access_files]            ;No access dirs collection
call ExternalUnlockAntiWriteLockCollection

push FULL_LOCK
push [dno_access_files2]          ;No access file  new-old size dword collection
call ExternalUnlockAntiWriteLockCollection

push FULL_LOCK
push [no_access_files2]           ;No access files collection
call ExternalUnlockAntiWriteLockCollection

;merge the AntiWriteLockCollection with the normal collection

push [dDeleted_files]
call MergeAntiWriteLockCollectionWithCollection

push [Deleted_files]
call MergeAntiWriteLockCollectionWithCollection


push [dno_access_files]           ;No access dirs collection
call MergeAntiWriteLockCollectionWithCollection

push [no_access_files]            ;No access dirs collection
call MergeAntiWriteLockCollectionWithCollection

push [dno_access_files2]          ;No access file  new-old size dword collection
call MergeAntiWriteLockCollectionWithCollection

push [no_access_files2]           ;No access files collection
call MergeAntiWriteLockCollectionWithCollection

mov [StopDirLoop],0
mov [Done],1
mov [Working],0
ret FunctionReturnFactor

;------------------------------------------------------------------------------
;This function is called when a new file is found in the (sub)directory
;input:
;    1:Path string (Unicode)
;    2:WIN32_FIND_DATA (unicode)
;    3:param

;output:
;       None
;------------------------------------------------------------------------------
countFiles:
call denter

  mov pebx,pdword [pebp+psize*4] ;param
  mov pecx,pdword [pebx+psize*4]

  mov pesi,pdword [pebp+psize*3]
  add pesi,0x2C
  jpcxz dontCopyFilename

    mov pedx,pecx
    sub pedx,2 ;get lock byte
    tryLockDontWait pedx,dontCopyFilename

    push pesi
    push pecx
    call strcpyW

    mov byte [pedx],0 ;clear lock

  dontCopyFilename:

    mov pecx,pdword [pebx+psize*2]  ;Total counted files
    jpcxz noTotalFilesCounter
    if used is32
      push ecx

      mov ecx,1
      push pdword [pebx+psize*2]
      call AddToQword

      pop ecx
    else
      inc qword [pecx]
    end if

    noTotalFilesCounter:
    mov pedx,pdword [pebp+psize*3]  ;WIN32_FIND_DATA
    mov pecx,pdword [pebx]  ;Total counted size
    jpcxz noTotalSizeCounter

    if used is32
      push dword [pedx+0x1C] ;nFileSizeHigh
      push dword [pedx+0x20] ;nFileSizeLow
      mov ecx,esp

      push pdword [pebx]
      call AddQwordToQword

      add esp,0x8

    else
      mov rax,qword [pedx+0x1C]  ;{nFileSizeHigh, nFileSizeLow}
      rol rax,32                 ;swap nFileSizeHigh and nFileSizeLow

      ;xor rax,rax
      ;xor rsi,rsi
      ;mov eax,dword [pedx+0x1C]
      ;shl rax,32
      ;mov esi,dword [pedx+0x20]
      ;add rax,rsi
      add qword [pecx],rax
    end if

    noTotalSizeCounter:
    ;Create same path to the compaired path
    mov pesi,pdword [pebx+psize*6] ;Base path
    mov pedi,pdword [pebx+psize*7] ;Compair path
    push pesi
    call strlenW

    mov pesi,pdword [pebp+psize*2] ;current path
    add pesi,peax

    mov peax,pdword [pebp+psize*3]
    add peax,0x2C ;current file

    push 0
    push peax
    push 0x2
    push pesi
    push 0x2
    push pedi
    push 0x2
    call pushBigPathsAllowed__countFiles

  db '\\?\',0

  pushBigPathsAllowed__countFiles     :
    push 0x1
    call CreateOneString
    mov pesi,peax ;compaired path


    ;check if script exists
    push 0x0 ;READ
    push pesi
    call openFileHandle
    mov pedx,peax

    inc peax
    jnz fileExists ;file exists, lets compair

      ;register removed or added files
      mov pedi,pdword [pebp+psize*3]  ;WIN32_FIND_DATA

      if used is64
        mov rax,qword [pedi+0x1C] ;{nFileSizeHigh, nFileSizeLow}
        rol rax,32                ;swap nFileSizeHigh and nFileSizeLow
        push rax                  ;push new size (for registerFile)
      else
        push dword [pedi+0x1C]  ;new size nFileSizeHigh (for registerFile)
        push dword [pedi+0x20]  ;new size nFileSizeLow (for registerFile)
       end if

      if used is32
         push 0 ;x86: nFileSizeHigh (for registerFile)
      end if
      push 0 ;old size (full on x64, x86: nFileSizeLow (for registerFile)



      test dword [pebx+psize*8],COMPAIR_REGISTER_DELETED
      jz doNotSwitchNewOld

        ;swap old and new if file is deleted
        if used is32
          mov eax,dword [esp]      ;get low (new)
          xchg eax,dword [esp+0x8] ;swap low (old) = low (new)
          mov dword [esp],eax ;    ;swap low (new) = low (old)

          mov eax,dword [esp+0x4]  ;get high (new)
          xchg eax,dword [esp+0xC] ;swap high (old) = high (new)
          mov dword [esp+0x4],eax  ;swap high (new) = high (old)
        else
          mov rax,qword [rsp]      ;get new qword
          xchg qword [rsp+8],rax   ;swap old = new
          mov qword [rsp],rax      ;new = old
        end if


      doNotSwitchNewOld:

      mov peax,pdword [pebp+psize*3]
      add peax,0x2C ;current file
      push peax ;current file

      push pdword [pebx+psize*6] ;Base path
      call strlenW
      mov pedi,pdword [pebp+psize*2] ;current path
      add pedi,peax

      push pedi ;current path
      ;push pesi ;filepath
      push pdword [pebx+psize*(9+5)]   ;Added files-size collection
      mov pedi,pdword [pebx+psize*9]   ;Added files collection
      call doesFileNotExist
      test eax,eax
      jnz fileIsAdded

        mov pedi,[pebx+psize*(10+5)]   ;No access files-size collection
        mov pdword [pesp],pedi
        mov pedi,[pebx+psize*10]       ;No access files collection

    fileIsAdded:
      push pedi

    AddFileToCollection:
    call registerFile
    jmp endCountFiles

  fileExists:
    mov pedi,pdword [pebx+psize*8] ;flags
    test edi,COMPAIR_MODIFIED_SIZE
    jz doNotCompairSize

    if used is32
      push 0
      push 0
      mov ecx,esp
    end if

    if used is32
      push ecx
    end if
    push pedx
    call getfilesizeLongLong
    if used is64
      mov rcx,rax
    end if

    mov pedi,pdword [pebp+psize*3]  ;WIN32_FIND_DATA

    if used is64
      mov rax,qword [pedi+0x1C]
      rol rax,32

      ;xor rax,rax
      ;mov eax,dword [pedi+0x1C]
      ;shl rax,32
      ;mov edi,dword [pedi+0x20]
      ;add rax,rdi
      cmp rax,rcx
    else
      push dword [pedi+0x1C] ;nFileSizeHigh
      push dword [pedi+0x20] ;nFileSizeLow
      mov edi,esp   ;Long long of current file

      push edi       ;Long long of current file
      push ecx       ;long long of path2 file
      call cmpQword
     end if
     jz sizeIfNotDifferent ;size is not different

     if used is64
       push rax ;push ;push new size (for registerFile)
       push rcx ;push old size (for registerFile)
     else
       push dword [edi+4] ;new size nFileSizeHigh (for registerFile)
       push dword [edi] ;new size nFileSizeLow (for registerFile)
       push dword [ecx+4] ;old size nFileSizeHigh (for registerFile)
       push dword [ecx] ;old size nFileSizeLow (for registerFile)
     end if

     push pdword [pebx+psize*6] ;Base path
     call strlenW
     mov pedi,pdword [pebp+psize*2] ;current path
     add pedi,peax

     mov peax,pdword [pebp+psize*3] ;WIN32_FIND_DATA
     add peax,0x2C ;current file

     ;register removed or added files
     push peax ;current file
     push pedi ;current path
     ;--push pesi ;filepath
     push pdword [pebx+psize*(11+5)] ;modified size files (with the new and old size) collection
     mov peax,pdword [pebx+psize*11] ;modified size files collection
     push peax
     call registerFile
     jmp endCountFiles

  sizeIfNotDifferent:
     if used is32
       add esp,psize*4
     end if

  doNotCompairSize:
    mov pedi,pdword [pebx+psize*8] ;flags
    test edi,COMPAIR_MODIFIED_DATE
    jz noLastWriteTime

    if used is32
      push 0
      push 0
      mov ecx,esp
      push ecx
    end if
    push pedx
    call getRawLastWriteTime
    test peax,peax
    jz noLastWriteTime

      mov pedi,pdword [pebp+psize*3]  ;WIN32_FIND_DATA

      if used is32
        add edi,5*4                   ;pointer to ftLastWriteTime
        push edi
        push ecx
        call cmpQword
      else
        cmp qword [rdi+5*4],rax       ;LastWriteTime==LastWriteTime
      end if
      jz lastWriteTimeEqual

      if used is64
        push qword [rdi+5*4] ;push new LastWriteTime (for registerFile)
        push rax             ;push old LastWriteTime (for registerFile)
      else
        push dword [edi+4]   ;push new LastWriteTime nFileSizeHigh (for registerFile)
        push dword [edi]     ;push new LastWriteTime nFileSizeLow (for registerFile)
        push dword [ecx+4]   ;push old LastWriteTime nFileSizeHigh (for registerFile)
        push dword [ecx]     ;push old LastWriteTime nFileSizeLow (for registerFile)
      end if

      push pdword [pebx+psize*6] ;Base path
      call strlenW
      mov pedi,pdword [pebp+psize*2] ;current path
      add pedi,peax

      mov peax,pdword [pebp+psize*3] ;WIN32_FIND_DATA
      add peax,0x2C ;current file

      ;register removed or added files
      push peax ;current file
      push pedi ;current path
      ;--push pesi ;filepath
      push pdword [pebx+psize*(12+5)] ;modified timedate files (new and old TimeDate) collection
      mov peax,pdword [pebx+psize*12] ;modified timedate files collection
      push peax
      call registerFile

      ;Also save the size of the file
      cmp pdword [pebx+psize*21],0
      je dontSaveSize

        mov pedi,pdword [pebp+psize*3]  ;WIN32_FIND_DATA
        ;add qword old  to dword collection (or low part of qword old on x86)
        push 0
        push pdword [pebx+psize*21]
        call AddDword

        ;add qword new to dword collection (or high part of qword old on x86)
        if used is64
          mov rax,qword [pedi+0x1C]
          rol rax,32
          push rax
        else
          push 0
        end if
        push pdword [pebx+psize*21]
        call AddDword

        if used is32
          ;add low part of qword new on x86)
          push dword [pedi+0x20]
          push pdword [pebx+psize*21]
          call AddDword

          ;add high part of qword new on x86)
          push dword [pedi+0x1C]
          push pdword [pebx+psize*21]
          call AddDword
        end if
      dontSaveSize:
      jmp endCountFiles

  lastWriteTimeEqual:
     if used is32
       add esp,psize*2
     end if

  noLastWriteTime:
    mov pedi,pdword [pebx+psize*8] ;flags
    test edi,COMPAIR_MODIFIED_ATTRIBUTES
    jz endCountFiles

    ;Get the attributes of the file in path2
    push pesi
    call getFileAttributes
    ;remove flags that are not set by this program
    mov ecx,ATTRIBUTES_NOT_SET_BY_SetFileAttributes
    and ecx,eax
    xor eax,ecx

    mov pecx,pdword [pebp+psize*3]  ;WIN32_FIND_DATA

    mov edi,ATTRIBUTES_NOT_SET_BY_SetFileAttributes
    and edi,dword [pecx]
    xor edi,dword [pecx]

    cmp eax,edi ;current file attributes
    jz fileAttributesNoDifferent ;endCountFiles

      if used is64
        mov ecx,dword [pecx] ;new file attributes
        push rcx             ;push new file attributes
        push rax             ;push old file attributes
      else
        mov ecx,dword [pecx] ;new file attributes
        push 0                  ;new file attributesHigh (for registerFile)
        push ecx                ;new file attributesLow (for registerFile)
        push 0                  ;old file attributesHigh (for registerFile)
        push eax                ;old file attributesLow (for registerFile)
      end if

      push pdword [pebx+psize*6] ;Base path
      call strlenW
      mov pedi,pdword [pebp+psize*2] ;current path
      add pedi,peax

      mov peax,pdword [pebp+psize*3] ;WIN32_FIND_DATA
      add peax,0x2C ;current file

      ;register removed or added files
      push peax ;current file
      push pedi ;current path
      ;-push pesi ;filepath
      push pdword [pebx+psize*(13+5)] ;modified attributes files (sizes) collection
      mov peax,pdword [pebx+psize*13] ;modified attributes files collection
      push peax
      call registerFile
      jmp endCountFiles

fileAttributesNoDifferent:
endCountFiles:
push pesi
call FreeMemLocation

mov peax,pedx
inc peax
jz noFileHandleToClose

  push pedx
  call closefile

noFileHandleToClose:
call dleave
ret psize*3


;------------------------------------------------------------------------------
;This function adds comobination of current path with file to the collection and than the compairing path
;input:
;    1:Collection
;    2:Dword collection
;    3:Dir
;    4:File
;    if used is64
;      5:qword old
;      6:qword new
;    else
;      5:low part of qword old
;      6:high part of qword old
;      7:low part of qword new
;      8:high part of qword new
;    end if

;output:
;       None
;------------------------------------------------------------------------------
registerFile:
cmp pdword [pesp+psize*1],0
je __endRegisterFile

cmp pdword [pesp+psize*2],0
je __endRegisterFile
call denter

    ;add qword old  to dword collection (or low part of qword old on x86)
    push pdword [pebp+psize*6]
    push pdword [pebp+psize*3]
    call AddDword

    ;add qword new to dword collection (or high part of qword old on x86)
    push pdword [pebp+psize*7]
    push pdword [pebp+psize*3]
    call AddDword

    if used is32
      ;add low part of qword new on x86)
      push pdword [pebp+psize*8]
      push pdword [pebp+psize*3]
      call AddDword

      ;add high part of qword new on x86)
      push pdword [pebp+psize*9]
      push pdword [pebp+psize*3]
      call AddDword
    end if

    ;add dir to collection
    push getSlicedMemory
    push pdword [pebp+psize*4]
    push pdword [pebp+psize*2]
    call AddStringExW ;AddStringW

    ;add file to collection
    push getSlicedMemory
    push pdword [pebp+psize*5]
    push pdword [pebp+psize*2]
    call AddStringExW ; AddStringW

call dleave
__endRegisterFile:
if used is32
  ret psize*8
else
  ret psize*6
end if

;------------------------------------------------------------------------------
;This function is called when a new directory is found in the (sub)directory
;input:
;    1:Path string (Unicode)
;    2:subdirectory string (unicode)
;    3:Full path string (unicode)
;    4:param
;    5:flags:
;      0x1 = first time eneterd this directory

;output:
;       None
;------------------------------------------------------------------------------
newSubdir:
call denter
  mov pedx,pdword [pebp+psize*5] ;param

  test pdword [pebp+psize*6],1 ;test is this is the first time entering this directory
  jne directoryAlreadyCounted

    mov pecx,pdword [pedx+psize*1] ;Total counted dirs
    jpcxz noTotalDirsCounter
    if used is32
      mov ecx,1
      push pdword [pedx+psize*1]
      call AddToQword
      else
      inc qword [pecx]
    end if
    noTotalDirsCounter:
  directoryAlreadyCounted:

  mov pecx,pdword [pedx+psize*3] ;Total counted dirs
  jpcxz noPathToCopy

    mov pesi,pecx
    sub pesi,2 ;get lock byte
    tryLockDontWait pesi,noPathToCopy

    push pdword [pebp+psize*4]   ;Full path
    push pecx
    call strcpyW

    mov byte [pesi],0 ;clear lock

  noPathToCopy:

call dleave
ret psize*5

;------------------------------------------------------------------------------
;This function is called when a directory cannot be accessed
;input:
;    1:Path string (Unicode)
;    2:subdirectory string (unicode)
;    3:Full path string (unicode)
;    4:param

;output:
;       None
;------------------------------------------------------------------------------
errDir:
call denter

  mov pebx,pdword [pebp+psize*5] ;param
  mov pecx,pdword [pebx+psize*5] ;Total unable to open directories
  jpcxz noTotalUnableToOpenDirectories
  if used is32
    mov ecx,1
    push pdword [pebx+psize*5]
    call AddToQword
    else
    inc qword [pecx]
  end if

noTotalUnableToOpenDirectories:
  cmp pdword [pebx+psize*19],0
  je noNoAccessDirCollection
    ;register unaccesable file
    mov pedx,pdword [pebp+psize*5] ;param

    if used is32
       push 0 ;x86: nFileSizeHigh (for registerFile)
    end if
    push 0 ;new size (full on x64, x86: nFileSizeLow (for registerFile)

    if used is32
       push 0 ;x86: nFileSizeHigh (for registerFile)
    end if
    push 0 ;old size (full on x64, x86: nFileSizeLow (for registerFile)

    call pushEmptyString ;current file
      
      du '',0

    pushEmptyString:
    push pdword [pebx+psize*6] ;Base path
    call strlenW
    mov pedi,pdword [pebp+psize*4] ;current path
    add pedi,peax
    push pedi ;current path
    push pdword [pebx+psize*20]   ;No access dirs size collection
    push pdword [pebx+psize*19]   ;No access dirs collection
    call registerFile
  noNoAccessDirCollection:

call dleave
ret psize*4

;    1:Collection
;    2:Dword collection
;    3:Dir
;    4:File