;------------------------------------------------------------------------------
;This function loops through the directory and sub-sirectories and calls a ball-back function.
;input:
;    1:Path string (UTF-16)
;    2:Callback function for every file in this and subdirectories (Path, WIN32_FIND_DATA, param)
;    3:Callback function for every directory in this and subdirectorie(s)  (Path, subdir, param, flags)
;      Flags:
;        0x1 = first time eneterd this directory (boolean)
;    4:Callback function for every directory in this and subdirectorie(s)  that cannot be accesed (Path, subdir, param)
;    5:param
;    6:
;       0: dirs and later call itself with 1
;       1: files only
;       2: both
;    7:stop pointer. Is set to 0 when this function is called, when set to 1, this function will stop and return

;output:
;       None
;------------------------------------------------------------------------------


StartDirectoryLoop:
  call denter

  push pdword [pebp+psize*8] ;stop pointer
  push pdword [pebp+psize*7] ;Files and then Dirs
  push pdword [pebp+psize*6] ;Param
  push pdword [pebp+psize*5] ;Path error callback
  push pdword [pebp+psize*4] ;Path  callback
  push pdword [pebp+psize*3] ;File callback
  push 0 ;no subdirectory
  push pdword [pebp+psize*2] ;no path
  push endStartDirectoryLoop ;push return pointer (instead of using call)
  call denter

  mov pesi,[pebp+psize*2]

  push 0

  call pushStarDotStar__Start

db '*.*',0

  pushStarDotStar__Start:
    push 0x1

    push pesi
    call strlenW
    cmp word [pesi+peax-2],'\'
    je patHasSlash__Start

    cmp word [pesi+peax-2],'/'
    je patHasSlash__Start

    call pushSlashOnStack__Start

  db '\',0

  pushSlashOnStack__Start:
    push 0x1

    patHasSlash__Start:
    push pesi
    push 0x2
    call pushBigPathsAllowed

  db '\\?\',0

  pushBigPathsAllowed:
    push 0x1
    call CreateOneString
  jmp __FirstFolderStart

  endStartDirectoryLoop:
    call dleave
ret psize*7

;------------------------------------------------------------------------------
;This function counts the size of a certain subdirectory.
;input:
;    1:Path string (UTF-16)
;    2:Subdirectory string (UTF-16)
;    3:Callback function for every file in this and subdirectories (Path, WIN32_FIND_DATA, param)
;    4:Callback function for every directory in this and subdirectorie(s)  (Path, subdir, Full path, param)
;    5:Callback function for every directory in this and subdirectorie(s)  that cannot be accesed (Path, subdir, Full path, param)
;    6:param
;    7:
;       0: dirs and later call itself with 1
;       1: files only
;       2: both

;    8:stop pointer. When set to 1, this function will stop and return

;output:
;       None
;------------------------------------------------------------------------------

LoopThroughSubDirectory:
  call denter
    push 0
    call pushStarDotStar__LoopThroughSubDirectory

  db '*.*',0

  pushStarDotStar__LoopThroughSubDirectory:
    push 0x1
    call pushSlash__LoopThroughSubDirectory

  db '\',0

  pushSlash__LoopThroughSubDirectory:
    push 0x1
    push pdword [pebp+psize*3]
    push 0x2
    push pdword [pebp+psize*2]
    push 0x2
    call CreateOneString

  __FirstFolderStart:
    mov pebx,peax;dword [ebp-0x4]

    ;Create WIN32_FIND_DATA
    ;push 318  ;for ASCII
    push 592   ;for UNICODE
    call GetMemLocation
    mov pedx,peax

  __redoLoopThroughSubDirectory:

    if used is32
      push pedx
    end if

    push pedx
    push pebx
    call FindFirstFile_win

    push peax

    ;Remove *.*
    push pebx
    call strlenW
    mov word [pebx+peax-6],0

    pop pecx
    if used is32
      pop pedx
    end if

    inc pecx  ; FindFirstFile returns -1 if failed, add 1 and you get 0
    jz errOpeningDirectory
    dec pecx

    cmp pdword [pebp+psize*5],0   ;check if there is a callback function for new directories
    jz StartFilePathLoop

    cmp pdword [pebp+psize*8],1  ;check if files only
    je StartFilePathLoop

    mov peax,pebx
    cmp dword [pebx],0x05C005C
    jne noBigPathsPrefix

      add peax,4*2

  noBigPathsPrefix:
     push 0x1                  ;Flag 0x1 = first time eneterd this directory
     push pdword [pebp+psize*7] ;Param
     push peax                  ;Full path
     push pdword [pebp+psize*3] ;Sub-dir

     mov peax,pdword [pebp+psize*2]
     cmp dword [peax],0x05C005C
     jne noBigPathsPrefixinBasePath

        add peax,4*2

    noBigPathsPrefixinBasePath:
     push peax                  ;Path
     call pdword [pebp+psize*5] ;callback when a new directory is being entered

    StartFilePathLoop:
      mov pesi,pedx
      add pesi,0x2C

      cmp word [pesi],'.';0x2E
      jne isNotParentOrCurrentDir

      add pesi,2
      cmp word [pesi],0x0
      je NextStartFilePathLoop

      cmp word [pesi],'.';0x2E
      jne isNotParentOrCurrentDir

      add pesi,2
      cmp word [pesi],0x0
      je NextStartFilePathLoop

    isNotParentOrCurrentDir:
      mov eax,dword [pedx]
      and eax,16
      jnz FoundFileIsPath

      cmp pdword [pebp+psize*8],1  ;check if files only
      je doHandleFile
      cmp pdword [pebp+psize*8],2  ;check if dirs and files in 1
      jne NextStartFilePathLoop

    doHandleFile:
      mov pesi,pedx
      add pesi,0x2C

      cmp pdword [pebp+psize*4],0
      jz doNotCallNewFileCallback

        mov peax,pebx
        cmp dword [pebx],0x05C005C
        jne noBigPathsPrefix__newFile

          add peax,4*2

      noBigPathsPrefix__newFile:
         push pdword [pebp+psize*7] ;Param
         push pedx                  ;WIN32_FIND_DATA
         push peax                  ;Path
         call pdword [pebp+psize*4]

    doNotCallNewFileCallback:
    jmp NextStartFilePathLoop

    FoundFileIsPath:
      mov pesi,pdword [pebp+psize*8]    ;dirs or files
      test pesi,pesi ;dirs, then files
      jz doLoopThoughDir

      cmp pesi,2
      jne NextStartFilePathLoop

    doLoopThoughDir:
      ;---If Is path
      mov pesi,pedx
      add pesi,0x2C

      push pdword [pebp+psize*9] ;stop pointer
      push 0                     ;dirs and then files
      cmp pdword [pebp+psize*8],2
      jne dontetDirsAndFilesTogether

        mov pdword [pesp],2

    dontetDirsAndFilesTogether:
      push pdword [pebp+psize*7] ;Param
      push pdword [pebp+psize*6] ;Path error callback
      push pdword [pebp+psize*5] ;Path  callback
      push pdword [pebp+psize*4] ;File callback
      push pesi ;subDir
      push pebx ;dir
      call LoopThroughSubDirectory

    cmp pdword [pebp+psize*5],0   ;check if there is a callback function for new directories
    jz NextStartFilePathLoop

    cmp pdword [pebp+psize*8],2  ;check if dirs and files in 1
    jne NextStartFilePathLoop

    mov peax,pebx
    cmp dword [pebx],0x05C005C
    jne noBigPathsPrefix__AftherSubdir

      add peax,4*2

  noBigPathsPrefix__AftherSubdir:

     push 0x0                   ;Flag 0x1 = first time eneterd this directory
     push pdword [pebp+psize*7] ;Param
     push peax                  ;Full path
     push pdword [pebp+psize*3] ;Sub-dir

     mov peax,pdword [pebp+psize*2]
     cmp dword [peax],0x05C005C
     jne noBigPathsPrefixinBasePath__AftherSubdir

        add peax,4*2

    noBigPathsPrefixinBasePath__AftherSubdir:
     push peax                  ;Path
     call pdword [pebp+psize*5] ;callback when a new directory is being entered

   NextStartFilePathLoop:
     mov peax,pdword [pebp+psize*9]
     cmp byte[peax],1
     je endLoopThroughSubDirectory

     if used is32
       push pedx
       push pecx
     end if

     push pedx
     push pecx
     call FindNextFile_win


     if used is32
       pop pecx
       pop pedx
     end if

     test peax,peax
   jnz StartFilePathLoop

jmp endLoopThroughSubDirectory

errOpeningDirectory:
    cmp pdword [pebp+psize*6],0   ;check if there is a callback function for no access directories
    jz endLoopThroughSubDirectory

    mov peax,pebx
    cmp dword [pebx],0x05C005C
    jne noBigPathsPrefix__forErrDir

      add peax,4*2

  noBigPathsPrefix__forErrDir:
     push pdword [pebp+psize*7] ;Param
     push peax                  ;Full path
     push pdword [pebp+psize*3] ;Sub-dir

     mov peax,pdword [pebp+psize*2]
     cmp dword [peax],0x05C005C
     jne noBigPathsPrefixinBasePath__forErrDir

        add peax,4*2

    noBigPathsPrefixinBasePath__forErrDir:
     push peax                  ;Path
     call pdword [pebp+psize*6] ;callback when a directory cannot be accessed

 endLoopThroughSubDirectory:
  test pecx,pecx
  jz noFindToClose

    if used is32
      push pedx
    end if

    push pecx
    call FindClose_win
    xor ecx,ecx

    if used is32
      pop pedx
    end if

noFindToClose:
  cmp pdword [pebp+psize*8],0  ;check if dirs, then directory
  jne dontRedoLoopThroughSubDirectory

  mov pedi,pebx
  push pebx
  call strlenW
  add pedi,peax
  mov dword [pedi],0x002E002A
  mov dword [pedi+4],0x002A

  mov pdword [pebp+psize*8],1 ;now do files
    jmp __redoLoopThroughSubDirectory

dontRedoLoopThroughSubDirectory:
  push pebx
  call FreeMemLocation

  push pedx
  call FreeMemLocation
  call dleave
ret psize*8

