
;Copy OneOnOne 1.00
;Copyright (c) 2022, Tycho Veltmeijer.
;All rights reserved.

;------------------------------------------------------------------------------
;This function will transfer structure FILETIME structure to a readable string (format orig-date in RFC 822)
;input:
;    1:FILETIME Qword (x64), FILETIME structure (x86)
;
;output:
;    EAX=> pointer to ASCI String

;------------------------------------------------------------------------------
if used FileTimeToString
FileTimeToString:
call denter

sub pesp,0x20 ;make space for  TIMEDATE structure
mov pedi,pesp

push pedi                   ;TIMEDATE
push pdword [pebp+psize*2]  ;FILETIME
call getTimeDateFromFileTime

push 0x60
call GetMemLocation
mov pesi,peax

push pesi                   ;output buffer
push pedi                   ;TIMEDATE
call SystemTimeToString

mov peax,pesi

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will transfer structure FILETIME structure to a readable string (format orig-date in RFC 822)
;input:
;    1:FILETIME Qword (x64), FILETIME structure (x86)
;    2:buffer
;
;output:
;    EAX=> end of the output buffer, with zero byte

;------------------------------------------------------------------------------
if used FileTimeToStringBuf
FileTimeToStringBuf:
call denter

sub pesp,0x20 ;make space for  TIMEDATE structure
mov pedi,pesp

push pedi                   ;TIMEDATE
push pdword [pebp+psize*2]  ;FILETIME
call getTimeDateFromFileTime

push pdword [pebp+psize*3]  ;output buffer
push pedi                   ;TIMEDATE
call SystemTimeToString

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will transfer structure TIMEDATE to a readable string (format orig-date in RFC 822)
;input:
;    1:TIMEDATE
;    2:output buffer (0x60 bytes)
;
;output:
;    EAX=> end of the output buffer, with zero byte
;------------------------------------------------------------------------------
if used SystemTimeToString
SystemTimeToString:
call denter

mov pesi,pdword [pebp+psize*2] ;TIMEDATE
mov pedi,pdword [pebp+psize*3] ;output buffer
mov ebx,10                     ;set base (for internalIntToStr)
xor ecx,ecx                    ;clear ECX

mov cx,word [pesi+6] ;lower word ECX =>dayOfTheWeek
call GetDay          ;Get 4 chars with name and comma
stosd                ;store the four chars in the output buffer

mov al,0x20
stosb                ;store space in output buffer

xor eax,eax          ;zero eax
mov ax,word [pesi+8] ;lower word EAX =>wDay

call addZerDigit     ;add '0' if ax<10
call internalIntToStr;call internal intToStr

mov al,0x20
stosb                ;store space in output buffer

mov cx,word [pesi+4] ;lower word ECX =>wMonth
call GetMonth        ;Get 3 chars with month name + space
stosd                ;store the four chars in the output buffer

xor eax,eax          ;zero EAX
lodsd                ;Get wYear
call internalIntToStr;call internal intToStr

mov al,0x20
stosb                ;store space in output buffer

add pesi,6           ;set pointer on wHour
xor eax,eax          ;zero EAX
lodsw                ;get wHour
call addZerDigit     ;add '0' if ax<10
call internalIntToStr;call internal intToStr

mov al,0x3A
stosb                ;store ':'

xor eax,eax          ;zero EAX
lodsw                ;get wMinute
call addZerDigit     ;add '0' if ax<10
call internalIntToStr;call internal intToStr

mov al,0x3A
stosb                ;store ':'

xor eax,eax          ;zero EAX
lodsw                ;get wSeconds
call addZerDigit     ;add '0' if ax<10
call internalIntToStr;call internal intToStr

mov eax,' UTC'
stosd                ;store ' UTC'

xor al,al
stosb                ;store zero byte

mov peax,pedi

call dleave
ret psize*2

;------------------------------------------------------------------------------
;This function will add a zero digit to the buffer if source<10
;input:
;    ax => source
;    (R/E)di => output buffer
;------------------------------------------------------------------------------
addZerDigit:
push peax

cmp ax,10
jae noForcedZero

  mov al,0x30 ;'0'
  stosb       ;Store first zero char

noForcedZero:
pop peax
ret

;------------------------------------------------------------------------------
;This function will transfer day number on ecx to a 3 char array with the name of the day + the "," char
;input:
;    ECX=>DAY
;output:
;    EAX=> 3 letter day
;------------------------------------------------------------------------------
GetDay:
mov eax,0x2C6E7553 ;Sunday
jecxz endGetDay

mov eax,0x2C6E6F4D ;Monday
dec ecx
jz endGetDay

mov eax,0x2C657554 ;Tuesday
dec ecx
jz endGetDay

mov eax,0x2C646557 ;Wednesday
dec ecx
jz endGetDay

mov eax,0x2C756854 ;Thursday
dec ecx
jz endGetDay

mov eax,0x2C697246 ;Friday
dec ecx
jz endGetDay

mov eax,0x2C746153 ;Saturday

endGetDay:
ret

;------------------------------------------------------------------------------
;This function will transfer month number on ecx to a 3 char array with the name of the month + space
;input:
;    ECX=>DAY
;output:
;    EAX=> 3 letter day
;------------------------------------------------------------------------------
GetMonth:
mov eax,0x206E614A ;January
dec ecx
jz endGetMonth

mov eax,0x20626546 ;February
dec ecx
jz endGetMonth

mov eax,0x2072614D ;March
dec ecx
jz endGetMonth

mov eax,0x20727041 ;April
dec ecx
jz endGetMonth

mov eax,0x2079614D ;May
dec ecx
jz endGetMonth

mov eax,0x206E754A ;June
dec ecx
jz endGetMonth

mov eax,0x206C754A ;July
dec ecx
jz endGetMonth

mov eax,0x20677541 ;August
dec ecx
jz endGetMonth

mov eax,0x20706553 ;September
dec ecx
jz endGetMonth

mov eax,0x2074634F ;October
dec ecx
jz endGetMonth

mov eax,0x20766F4E ;November
dec ecx
jz endGetMonth

mov eax,0x20636544 ;December

endGetMonth:
ret
end if