function callApi(args, loadFunction, jsonResponse=true, param=null, errorfunction=null, uploadProgressFunction=null) {
  createRequest("api/", args, loadFunction, jsonResponse, param, errorfunction, uploadProgressFunction);
}

function createRequest(url, args, loadFunction, jsonResponse=true, param=null, errorfunction=null, uploadProgressFunction=null) {
  let request = setUpDefaultXMLRequest(errorfunction);
  let params="";
  
  if(uploadProgressFunction!=null) {
    request.upload.onprogress = uploadProgressFunction;
    request.upload.param = param;
  }

  params = "";
  for (let name in args) {
    if(params!="")
      params+="&";
    if(Array.isArray(args[name]) || (typeof args[name]=="object" && args[name]!=null))
      args[name] = JSON.stringify(args[name]);
    params+=encodeURIComponent(name)+'='+encodeURIComponent(args[name]);
  }
  /*request.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');*/
  
  
  request.open("GET", url + "?" + params, true);
 
  
  if(typeof jsonResponse == "boolean" && jsonResponse)
    request.responseType = "json";
  else if(typeof jsonResponse == "string" && jsonResponse)
    request.responseType = jsonResponse;
  request.onload = loadFunction;
  request.param = param;
  request.params = params;
  
  request.send(params);
  //setTimeout(callSendParam, 1000, request, params);
}

function setUpDefaultXMLRequest(errorfunction=null, retries=5, defaultTimeoutRetry=2000) {
  let xmlHttp = new XMLHttpRequest();
  xmlHttp.retries = retries;
  xmlHttp.defaultTimeoutRetry = defaultTimeoutRetry;
  
  if(errorfunction!=null) {
    xmlHttp.onerror = errorfunction;
    xmlHttp.onabort = errorfunction;
    xmlHttp.timeout = errorfunction;
  } else {
    xmlHttp.onerror = async function (e) {
      var message = "Error XMLHttp request...";
      if(!retryXmlHttp(this, message))
        logXmlHttpLog(this, message + "stopping");
    };
    xmlHttp.onabort = async function (e) {
      var message = "Abort XMLHttp request...";
      if(!retryXmlHttp(this,  message))
        logXmlHttpLog(this, message + "stopping");
    };
    xmlHttp.timeout = async function (e) {
      var message = "Timeout XMLHttp request...";
      if(!retryXmlHttp(this, message))
        logXmlHttpLog(this, message + "stopping");
    };
  }
  return xmlHttp;
}

function retryXmlHttp(http, message=null, timeOut=null) {
  if(http.retries!=0) {
    logXmlHttpLog(http, "(" + http.retries + ")" + message, true);
    
    if(timeOut==null)
      timeOut = http.defaultTimeoutRetry;
    
    http.retries--;
    http.open("POST", http.responseURL);
    setTimeout(callSendParam, http.defaultTimeoutRetr, http, http.params);
    return true;
  }
  
  return false;
}

function logXmlHttpLog(http, message, useLog=false) {
  var date = new Date();
  var dateStr = date.getDate() + "-" + month2digits(date.getMonth() + 1) + "-" + date.getUTCFullYear() + " ";
  
  if(message!=null) {
    if(useLog)
      log(dateStr + message);
    else
      debug(dateStr + message);
  }
}

function log(message) {
  console.log(message);
}

function debug(message) {
  console.debug(message);
}

function callSend(obj) {
  obj.send();
}

function callSendParam(obj, param) {
  obj.send(param);
}

function month2digits(month) { 
  return (month < 10 ? '0' : '') + month;
}